@@ -1,7 +1,7 @@
 /*
  * dumpseg.c - NILFS command of printing segment information.
  *
- * Copyright (C) 2007 Nippon Telegraph and Telephone Corporation.
+ * Copyright (C) 2007-2012 Nippon Telegraph and Telephone Corporation.
  *
  * This file is part of NILFS.
  *
@@ -15,13 +15,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with NILFS; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * Written by Koji Sato.
  *
- * Written by Koji Sato <koji@osrg.net>.
- *
- * Maintained by Ryusuke Konishi <ryusuke@osrg.net> from 2008.
+ * Maintained by Ryusuke Konishi <konishi.ryusuke@lab.ntt.co.jp> from 2008.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,6 +34,10 @@
 #include <unistd.h>
 #endif	/* HAVE_UNISTD_H */
 
+#if HAVE_ERR_H
+#include <err.h>
+#endif	/* HAVE_ERR_H */
+
 #if HAVE_STRING_H
 #include <string.h>
 #endif	/* HAVE_STRING_H */
@@ -46,12 +46,11 @@
 #include <time.h>
 #endif	/* HAVE_TIME_H */
 
-#include <errno.h>
 #include "nilfs.h"
 
 #ifdef _GNU_SOURCE
 #include <getopt.h>
-const static struct option long_option[] = {
+static const struct option long_option[] = {
 	{"help", no_argument, NULL, 'h'},
 	{"version", no_argument, NULL, 'V'},
 	{NULL, 0, NULL, 0}
@@ -71,14 +70,16 @@ const static struct option long_option[] = {
 
 static void dumpseg_print_block(struct nilfs_block *blk)
 {
+	__le64 *binfo = blk->b_binfo;
+
 	if (nilfs_block_is_data(blk)) {
 		printf("        vblocknr = %llu, blkoff = %llu, blocknr = %llu\n",
-		       (unsigned long long)le64_to_cpu(*(__le64 *)blk->b_binfo),
-		       (unsigned long long)le64_to_cpu(*((__le64 *)blk->b_binfo + 1)),
+		       (unsigned long long)le64_to_cpu(binfo[0]),
+		       (unsigned long long)le64_to_cpu(binfo[1]),
 		       (unsigned long long)blk->b_blocknr);
 	} else {
 		printf("        vblocknr = %llu, blocknr = %llu\n",
-		       (unsigned long long)le64_to_cpu(*(__le64 *)blk->b_binfo),
+		       (unsigned long long)le64_to_cpu(binfo[0]),
 		       (unsigned long long)blk->b_blocknr);
 	}
 }
@@ -86,11 +87,14 @@ static void dumpseg_print_block(struct nilfs_block *blk)
 static void dumpseg_print_block_super(struct nilfs_block *blk)
 {
 	if (nilfs_block_is_data(blk)) {
+		__le64 *binfo = blk->b_binfo;
+
 		printf("        blkoff = %llu, blocknr = %llu\n",
-		       (unsigned long long)le64_to_cpu(*(__le64 *)blk->b_binfo),
+		       (unsigned long long)le64_to_cpu(binfo[0]),
 		       (unsigned long long)blk->b_blocknr);
 	} else {
 		struct nilfs_binfo_dat *bid = blk->b_binfo;
+
 		printf("        blkoff = %llu, level = %d, blocknr = %llu\n",
 		       (unsigned long long)le64_to_cpu(bid->bi_blkoff),
 		       bid->bi_level,
@@ -170,7 +174,7 @@ int main(int argc, char *argv[])
 {
 	struct nilfs *nilfs;
 	__u64 segnum;
-	char *dev, *endptr, *progname;
+	char *dev, *endptr, *progname, *last;
 	void *seg;
 	ssize_t segsize;
 	int c, i, status;
@@ -180,10 +184,8 @@ int main(int argc, char *argv[])
 
 	opterr = 0;
 
-	if ((progname = strrchr(argv[0], '/')) == NULL)
-		progname = argv[0];
-	else
-		progname++;
+	last = strrchr(argv[0], '/');
+	progname = last ? last + 1 : argv[0];
 
 #ifdef _GNU_SOURCE
 	while ((c = getopt_long(argc, argv, "hV",
@@ -195,21 +197,18 @@ int main(int argc, char *argv[])
 		switch (c) {
 		case 'h':
 			fprintf(stderr, DUMPSEG_USAGE, progname);
-			exit(0);
+			exit(EXIT_SUCCESS);
 		case 'V':
 			printf("%s (%s %s)\n", progname, PACKAGE,
 			       PACKAGE_VERSION);
-			exit(0);
+			exit(EXIT_SUCCESS);
 		default:
-			fprintf(stderr, "%s: invalid option -- %c\n",
-				progname, optopt);
-			exit(1);
+			errx(EXIT_FAILURE, "invalid option -- %c", optopt);
 		}
 	}
 
 	if (optind > argc - 1) {
-		fprintf(stderr, "%s: too few arguments\n", progname);
-		exit(1);
+		errx(EXIT_FAILURE, "too few arguments");
 	} else {
 		strtoull(argv[optind], &endptr, DUMPSEG_BASE);
 		if (*endptr == '\0')
@@ -219,32 +218,30 @@ int main(int argc, char *argv[])
 	}
 
 	nilfs = nilfs_open(dev, NULL, NILFS_OPEN_RAW);
-	if (nilfs == NULL) {
-		fprintf(stderr, "%s: %s: cannot open NILFS\n", progname, dev);
-		exit(1);
-	}
+	if (nilfs == NULL)
+		err(EXIT_FAILURE, "cannot open NILFS on %s", dev ? : "device");
 
 	if (nilfs_opt_set_mmap(nilfs) < 0)
-		fprintf(stderr, "%s: cannot use mmap\n", progname);
+		warnx("cannot use mmap");
 
-	status = 0;
+	status = EXIT_SUCCESS;
 	for (i = optind; i < argc; i++) {
 		segnum = strtoull(argv[i], &endptr, DUMPSEG_BASE);
 		if (*endptr != '\0') {
-			fprintf(stderr, "%s: %s: invalid segment number\n",
-				progname, argv[i]);
-			status = 1;
+			warnx("%s: invalid segment number", argv[i]);
+			status = EXIT_FAILURE;
 			continue;
 		}
-		if ((segsize = nilfs_get_segment(nilfs, segnum, &seg)) < 0) {
-			fprintf(stderr, "%s: %s\n", progname, strerror(errno));
-			status = 1;
+		segsize = nilfs_get_segment(nilfs, segnum, &seg);
+		if (segsize < 0) {
+			warn(NULL);
+			status = EXIT_FAILURE;
 			goto out;
 		}
 		dumpseg_print_segment(nilfs, segnum, seg, segsize);
 		if (nilfs_put_segment(nilfs, seg) < 0) {
-			fprintf(stderr, "%s: %s\n", progname, strerror(errno));
-			status = 1;
+			warn(NULL);
+			status = EXIT_FAILURE;
 			goto out;
 		}
 	}
