@@ -13,10 +13,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
  * Written by Koji Sato.
  *
  * Maintained by Ryusuke Konishi <konishi.ryusuke@lab.ntt.co.jp> from 2008.
@@ -864,13 +860,11 @@ int nilfs_put_segment(struct nilfs *nilfs, void *segment)
 			(1UL << (le32_to_cpu(nilfs->n_sb->s_log_block_size) +
 				 NILFS_SB_BLOCK_SIZE_SHIFT));
 		return munmap(segment, segsize);
-	} else {
-#endif	/* HAVE_MUNMAP */
-		free(segment);
-		return 0;
-#ifdef HAVE_MUNMAP
 	}
 #endif	/* HAVE_MUNMAP */
+
+	free(segment);
+	return 0;
 }
 
 __u64 nilfs_get_segment_seqnum(const struct nilfs *nilfs, void *segment,
